package me.mrletsplay.bmareas;

import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.imageio.ImageIO;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.bluecolored.bluemap.api.BlueMapAPI;
import de.bluecolored.bluemap.api.gson.MarkerGson;
import de.bluecolored.bluemap.api.markers.MarkerSet;
import me.mrletsplay.bmareas.command.Icon;
import me.mrletsplay.bmareas.command.Point;
import me.mrletsplay.bmareas.command.area.AreaCommand;
import me.mrletsplay.bmareas.command.bmareas.BMAreasCommand;
import me.mrletsplay.bmareas.command.marker.MarkerCommand;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.minecraft.registry.RegistryKey;
import net.minecraft.world.World;

public class BMAreas implements ModInitializer {
	public static final String MOD_ID = "bmareas";

	public static final Logger LOGGER = LoggerFactory.getLogger("dynmapareas");

	private static final List<RegistryKey<World>> DIMENSIONS = List.of(
		World.OVERWORLD,
		World.NETHER,
		World.END
	);

	public static final Path
		DATA_PATH = Path.of("bmareas/data"),
		BLUEMAP_WEB_PATH = Path.of("bluemap/web"),
		ICONS_PATH = BLUEMAP_WEB_PATH.resolve("assets/icons");

	public static Map<UUID, List<Point>> points = new HashMap<>();
	public static BlueMapAPI markerAPI;
//	public static MarkerSet areaMarkerSet, markerMarkerSet;
	private static Map<RegistryKey<World>, MarkerSet>
		areaMarkerSets,
		markerMarkerSets;

	public static List<Icon> icons;

	@Override
	public void onInitialize() {
		LOGGER.info("HELLO");

		areaMarkerSets = new HashMap<>();
		markerMarkerSets = new HashMap<>();

		loadIcons();

		BlueMapAPI.onEnable(api -> {
			markerAPI = api;
			loadMarkers();
		});

		CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
			dispatcher.register(new BMAreasCommand().create());
			dispatcher.register(new AreaCommand().create());
			dispatcher.register(new MarkerCommand().create());
		});
	}

	public static MarkerSet getAreaMarkerSet(World world) {
		return areaMarkerSets.get(world.getRegistryKey());
	}

	public static MarkerSet getMarkerMarkerSet(World world) {
		return markerMarkerSets.get(world.getRegistryKey());
	}

	public static void loadIcons() {
		try {
			List<Path> iconFiles = Files.walk(ICONS_PATH)
				.filter(Files::isRegularFile)
				.filter(p -> p.getFileName().toString().endsWith(".png"))
				.toList();

			List<Icon> icons = new ArrayList<>();

			for(Path p : iconFiles) {
				try {
					BufferedImage img = ImageIO.read(p.toFile());
					String fileName = ICONS_PATH.relativize(p).toString();
					fileName = fileName.substring(0, fileName.length() - ".png".length());
					icons.add(new Icon(p, fileName, img.getWidth(), img.getHeight()));
				}catch(IOException e) {
					LOGGER.warn("Skipping invalid icon " + p.toString());
				}
			}

			BMAreas.icons = icons;
		} catch (IOException e) {
			icons = Collections.emptyList();
		}
	}

	private static MarkerSet loadMarkerSet(Path path, String defaultLabel) {
		if(!Files.exists(path)) return new MarkerSet(defaultLabel);

		try (BufferedReader reader = Files.newBufferedReader(path)) {
			return MarkerGson.INSTANCE.fromJson(reader, MarkerSet.class);
		} catch (IOException e) {
			LOGGER.error("Failed to load marker set at " + path, e);
			return null;
		}
	}

	public static void loadMarkers() {
		for(var key : DIMENSIONS) {
			Path dimPath = DATA_PATH.resolve(key.getValue().getNamespace() + "-" + key.getValue().getPath());
			try {
				Files.createDirectories(dimPath);
			} catch (IOException e) {
				LOGGER.error("Failed to create directory for dimension " + key, e);
				continue;
			}

			MarkerSet areaMarkerSet = loadMarkerSet(dimPath.resolve("areas.json"), "Player Areas");
			MarkerSet markerMarkerSet = loadMarkerSet(dimPath.resolve("markers.json"), "Player Areas");
			areaMarkerSets.put(key, areaMarkerSet);
			markerMarkerSets.put(key, markerMarkerSet);

			markerAPI.getWorld(key).get().getMaps().forEach(map -> {
				map.getMarkerSets().put("bmareas_areas", areaMarkerSets.get(key));
				map.getMarkerSets().put("bmareas_marker", markerMarkerSets.get(key));
			});
		}
	}

	private static void saveMarkerSet(Path path, MarkerSet markerSet) {
		try(BufferedWriter writer = Files.newBufferedWriter(path)) {
			MarkerGson.INSTANCE.toJson(markerSet, writer);
		} catch (IOException e) {
			LOGGER.error("Failed to save marker set at " + path, e);
		}
	}

	public static void saveMarkers() {
		for(var key : DIMENSIONS) {
			Path dimPath = DATA_PATH.resolve(key.getValue().getNamespace() + "-" + key.getValue().getPath());

			saveMarkerSet(dimPath.resolve("areas.json"), areaMarkerSets.get(key));
			saveMarkerSet(dimPath.resolve("markers.json"), markerMarkerSets.get(key));
		}
	}

//	// Taken from https://github.com/webbukkit/dynmap/blob/b8b1e8bd4e91f86e8017db8531c1e4faf37ce070/fabric-1.20/src/main/java/org/dynmap/fabric_1_20/FabricWorld.java#L39C5-L50C6
//	public static String getWorldName(World w) {
//		RegistryKey<World> rk = w.getRegistryKey();
//		if (rk == World.OVERWORLD) { // Overworld?
//			return w.getServer().getSaveProperties().getLevelName();
//		} else if (rk == World.END) {
//			return "DIM1";
//		} else if (rk == World.NETHER) {
//			return "DIM-1";
//		} else {
//			return rk.getValue().getNamespace() + "_" + rk.getValue().getPath();
//		}
//	}
}