package me.mrletsplay.bmareas.command.area;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import me.mrletsplay.bmareas.BMAreas;
import me.mrletsplay.bmareas.command.Corner;
import me.mrletsplay.bmareas.command.Point;
import me.mrletsplay.mrcore.command.event.CommandInvokedEvent;
import me.mrletsplay.mrcore.fabric.command.FabricCommand;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;
import net.minecraft.util.math.BlockPos;

public class AreaAddCornerCommand extends FabricCommand {

	public AreaAddCornerCommand() {
		super("addcorner");
		setDescription("Add a corner to your area");
		setUsage("/area addcorner [offset]");
		setTabCompleter(event -> {
			if(event.getArgs().length == 0) {
				return Arrays.stream(Corner.values())
						.map(c -> c.getName())
						.collect(Collectors.toList());
			}
			return Collections.emptyList();
		});
	}

	@Override
	public void action(CommandInvokedEvent event) {
		ServerPlayerEntity p = getSenderPlayer(event);
		if(p == null) {
			event.getSender().sendMessage("Player only!");
			return;
		}

		if(event.getArguments().length > 1) {
			sendCommandInfo(event.getSender());
			return;
		}

		Corner c = Corner.CENTER;
		if(event.getArguments().length == 1) {
			c = Corner.getByName(event.getArguments()[0]);
			if(c == null) {
				sendCommandInfo(event.getSender());
				return;
			}
		}

		BlockPos loc = p.getBlockPos();
		double x = loc.getX() + c.getDX();
		double z = loc.getZ() + c.getDZ();
		List<Point> points = BMAreas.points.getOrDefault(p.getUuid(), new ArrayList<>());
		points.add(new Point(p.getWorld(), x, z));
		BMAreas.points.put(p.getUuid(), points);
		p.sendMessage(Text.literal(String.format("§aAdded corner at (§7%s§a/§7%s§a)", x, z)));
	}

}
