package me.mrletsplay.bmareas.command.area;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.flowpowered.math.vector.Vector2d;

import de.bluecolored.bluemap.api.markers.ShapeMarker;
import de.bluecolored.bluemap.api.math.Color;
import de.bluecolored.bluemap.api.math.Shape;
import me.mrletsplay.bmareas.BMAreas;
import me.mrletsplay.bmareas.command.Point;
import me.mrletsplay.mrcore.command.event.CommandInvokedEvent;
import me.mrletsplay.mrcore.fabric.command.FabricCommand;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;

public class AreaDoneCommand extends FabricCommand {

	public AreaDoneCommand() {
		super("done");
		setDescription("Create your own area and add it to the map");
		setUsage("/area done \"<name>\" [color (#000000)]");

		setTabCompleter(event -> {
			if(event.getArgs().length == 0) {
				return Arrays.asList("My fancy area");
			}else if(event.getArgs().length == 1) {
				return Arrays.asList("#FF00FF");
			}
			return Collections.emptyList();
		});
	}

	@Override
	public void action(CommandInvokedEvent event) {
		ServerPlayerEntity p = getSenderPlayer(event);
		if(p == null) {
			event.getSender().sendMessage("Player only!");
			return;
		}

		if(event.getArguments().length < 1 || event.getArguments().length > 2) {
			sendCommandInfo(event.getSender());
			return;
		}

		String name = event.getArguments()[0].replace("\"", "");
		int colorInt = 0xFF0000;
		if(event.getArguments().length == 2) {
			try {
				String colStr = event.getArguments()[1];
				colorInt = Integer.parseInt(colStr.startsWith("#") ? colStr.substring(1) : colStr, 16);
			}catch(NumberFormatException e) {
				sendCommandInfo(event.getSender());
				return;
			}
		}

		List<Point> points = BMAreas.points.get(p.getUuid());
		if(points == null || points.size() < 2) {
			p.sendMessage(Text.literal("§cYou need to add at least 2 corners using §7/area addcorner"));
			return;
		}


		List<Vector2d> vectors = points.stream()
			.map(pt -> new Vector2d(pt.getX(), pt.getZ()))
			.toList();

		ShapeMarker shape = new ShapeMarker(name, new Shape(vectors), 64);
		shape.setColors(new Color(colorInt, 1f), new Color(colorInt, 0.3f));
		shape.setDepthTestEnabled(false);
//		AreaMarker a = BMAreas.areaMarkerSet.createAreaMarker(p.getUuid().toString() + "_" + Long.toHexString(System.nanoTime()), name, false, DynmapAreasMod.getWorldName(points.get(0).getWorld()), x, z, true);
//		a.setFillStyle(a.getFillOpacity(), colorInt);
//		a.setLineStyle(a.getLineWeight(), a.getLineOpacity(), colorInt);
		BMAreas.getAreaMarkerSet(p.getWorld()).put(p.getUuid().toString() + "_" + Long.toHexString(System.nanoTime()), shape);
		BMAreas.saveMarkers();
		BMAreas.points.remove(p.getUuid());
		p.sendMessage(Text.literal("§aArea created"));
	}

}
